@@ -1,12 +1,9 @@
-#!/usr/bin/env python2
-# -*- coding: utf-8 -*-
-#
-# mails.py
-#
-# Copyright 2011 - 2016 Patrick Ulbrich <zulu99@gmx.net>
+# Copyright 2011 - 2020 Patrick Ulbrich <zulu99@gmx.net>
+# Copyright 2020 Andreas Angerer
 # Copyright 2016, 2018 Timo Kankare <timo.kankare@iki.fi>
 # Copyright 2011 Leighton Earl <leighton.earl@gmx.com>
 # Copyright 2011 Ralf Hersel <ralf.hersel@gmx.net>
+# Copyright 2019 razer <razerraz@free.fr>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,7 +27,7 @@ import os
 import logging
 import hashlib
 
-from email.header import decode_header
+from email.header import decode_header, make_header
 from Mailnag.common.i18n import _
 from Mailnag.common.config import cfg_folder
 
@@ -39,13 +36,13 @@ from Mailnag.common.config import cfg_folder
 # Mail class
 #
 class Mail:
-	def __init__(self, datetime, subject, sender, id, account):
+	def __init__(self, datetime, subject, sender, id, account, flags):
 		self.datetime = datetime
 		self.subject = subject
 		self.sender = sender
-		self.account_name = account.name
-		self.account_id = account.get_id()
+		self.account = account
 		self.id = id
+		self.flags = flags
 
 
 #
@@ -70,7 +67,7 @@ class MailCollector:
 				logging.error("Failed to open mailbox for account '%s' (%s)." % (acc.name, ex))
 				continue
 
-			for folder, msg in acc.list_messages():
+			for folder, msg, flags in acc.list_messages():
 				sender, subject, datetime, msgid = self._get_header(msg)
 				id = self._get_id(msgid, acc, folder, sender, subject, datetime)
 			
@@ -82,7 +79,7 @@ class MailCollector:
 				# Also filter duplicates caused by Gmail labels.
 				if id not in mail_ids:
 					mail_list.append(Mail(datetime, subject, \
-						sender, id, acc))
+						sender, id, acc, flags))
 					mail_ids[id] = None
 
 			# leave account with notifications open, so that it can
@@ -93,34 +90,50 @@ class MailCollector:
 		
 		# sort mails
 		if sort:
-			mail_list = sort_mails(mail_list, sort_desc = True)
+			mail_list.sort(key = lambda m: m.datetime, reverse = True)
 		
 		return mail_list
 
 
 	def _get_header(self, msg_dict):
-		try:
-			content = self._get_header_field(msg_dict, 'From')
-			sender = self._format_header_field('sender', content)
-		except:
-			sender = ('', '')
-
-		try:
-			content = self._get_header_field(msg_dict, 'Subject')
-		except:
-			content = _('No subject')
-		try:
-			subject = self._format_header_field('subject', content)
-		except:
-			subject = ''
+		# Get sender
+		sender = ('', '')
 		
+		try:
+			content = self._get_header_field(msg_dict, 'From')
+			# get the two parts of the sender
+			addr = email.utils.parseaddr(content)
+			
+			if len(addr) != 2:
+				logging.warning('Malformed sender field in message.')
+			else:
+				sender_real = self._convert(addr[0])
+				sender_addr = self._convert(addr[1])
+			
+				sender = (sender_real, sender_addr)
+		except:
+			pass
+
+		# Get subject
+		try:
+			content = self._get_header_field(msg_dict, 'Subject')
+			subject = self._convert(content)
+		except:
+			subject = _('No subject')
+		
+		# Get date
 		try:
 			content = self._get_header_field(msg_dict, 'Date')
-			datetime = self._format_header_field('date', content)
+			
+			# make a 10-tupel (UTC)
+			parsed_date = email.utils.parsedate_tz(content)
+			# convert 10-tupel to seconds incl. timezone shift
+			datetime = email.utils.mktime_tz(parsed_date)
 		except:
 			logging.warning('Email date set to zero.')
 			datetime = 0
-			
+		
+		# Get message id
 		try:
 			msgid = self._get_header_field(msg_dict, 'Message-ID')
 		except:
@@ -140,49 +153,13 @@ class MailCollector:
 		
 		return value
 
-	
-	# format sender, date, subject etc.
-	def _format_header_field(self, field_name, content):
-		if field_name == 'sender':
-			# get the two parts of the sender
-			sender_real, sender_addr = email.utils.parseaddr(content)
-			sender_real = self._convert(sender_real)
-			sender_addr = self._convert(sender_addr)
-			# create decoded tupel
-			return (sender_real, sender_addr)
 
-		if field_name == 'date':
-			# make a 10-tupel (UTC)
-			parsed_date = email.utils.parsedate_tz(content)
-			# convert 10-tupel to seconds incl. timezone shift
-			return email.utils.mktime_tz(parsed_date)
-
-		if field_name == 'subject':
-			return self._convert(content)
-
-
-	# decode and concatenate multi-coded header parts
-	def _convert(self, raw_content):
-		# replace newline by space
-		content = raw_content.replace('\n',' ')
-		# workaround a bug in email.header.decode_header()
-		content = content.replace('?==?','?= =?')
-		# list of (text_part, charset) tupels
-		tupels = decode_header(content)
-		content_list = []
-		# iterate trough parts
-		for text, charset in tupels:
-			# set default charset for decoding
-			if charset == None: charset = 'latin-1'
-			# replace non-decodable chars with 'nothing'
-			content_list.append(text.decode(charset, 'ignore'))
-		
-		# insert blanks between parts
-		decoded_content = u' '.join(content_list)
-		# get rid of whitespace
-		decoded_content = decoded_content.strip()
-
-		return decoded_content
+	# return utf-8 decoded string from multi-part/multi-charset header text
+	def _convert(self, text):
+		decoded = decode_header(text)
+		if not decoded[0][1] or 'unknown' in decoded[0][1]:
+			decoded = [(decoded[0][0], 'latin-1')]
+		return str(make_header(decoded))
 
 
 	def _get_id(self, msgid, acc, folder, sender, subject, datetime):
@@ -217,20 +194,20 @@ class MailSyncer:
 		
 		# collect mails from given accounts
 		rcv_lst = MailCollector(self._cfg, accounts).collect_mail(sort = False)
-	
+		
 		# group received mails by account
 		tmp = {}
 		for acc in accounts:
 			tmp[acc.get_id()] = {}
 		for mail in rcv_lst:
-			tmp[mail.account_id][mail.id] = mail
+			tmp[mail.account.get_id()][mail.id] = mail
 	
 		# compare current mails against received mails
 		# and remove those that are gone (probably opened in mail client).
-		for acc_id in self._mails_by_account.iterkeys():
+		for acc_id in self._mails_by_account.keys():
 			if acc_id in tmp:
 				del_ids = []
-				for mail_id in self._mails_by_account[acc_id].iterkeys():
+				for mail_id in self._mails_by_account[acc_id].keys():
 					if not (mail_id in tmp[acc_id]):
 						del_ids.append(mail_id)
 						needs_rebuild = True
@@ -253,31 +230,11 @@ class MailSyncer:
 			for acc_id in self._mails_by_account:
 				for mail_id in self._mails_by_account[acc_id]:
 					self._mail_list.append(self._mails_by_account[acc_id][mail_id])
-			self._mail_list = sort_mails(self._mail_list, sort_desc = True)
+			self._mail_list.sort(key = lambda m: m.datetime, reverse = True)
 		
 		return self._mail_list
 
 
-#
-# sort_mails function
-#
-def sort_mails(mail_list, sort_desc = False):
-	sort_list = []
-	for mail in mail_list:
-		sort_list.append([mail.datetime, mail])
-	
-	# sort asc
-	sort_list.sort()
-	if sort_desc:
-		sort_list.reverse()
-
-	# recreate mail_list
-	mail_list = []
-	for mail in sort_list:
-		mail_list.append(mail[1])
-	return mail_list
-
-
 #
 # Memorizer class
 #
@@ -315,7 +272,7 @@ class Memorizer(dict):
 		
 		dat_file = os.path.join(cfg_folder, 'mailnag.dat')
 		with open(dat_file, 'w') as f:
-			for id, seen_flag in self.items():
+			for id, seen_flag in list(self.items()):
 				line = id + ',' + seen_flag + '\n'
 				f.write(line)
 
@@ -329,7 +286,7 @@ class Memorizer(dict):
 				self[m.id] = '0'
 				self._changed = True
 		
-		for id in self.keys():
+		for id in list(self.keys()):
 			found = False
 			for m in mail_list:			
 				if id == m.id:
